#include "headers.h"

/* Grammar
 *
 * Every sentence has three parts, in the following order:
 *
 *  rule_condition  separator  rule_action
 *
 *  The rule condition is what is to be tested, if successful the rule action
 * will be executed. The separator indicates which kind of action will be  
 * done by the rule_action. Although the real action is done in the separator,
 * we will stick with this terminology in this document.
 *
 *
 *
 */

typedef struct {
	char       *value;
	int        type;
} RULE_ACTION;


char    *process_rule   PROTO ((char *,int, ENVELOPE *));
int	 test_condition PROTO ((char *,int, ENVELOPE *));
int      test_in        PROTO ((char *,char *, ENVELOPE *, int));   
int      test_ni        PROTO ((char *,char *, ENVELOPE *, int));
int      test_not_in    PROTO ((char *,char *, ENVELOPE *, int));
int      test_not_ni    PROTO ((char *,char *, ENVELOPE *, int));
int      test_eq        PROTO ((char *,char *, ENVELOPE *, int));
int      test_not_eq    PROTO ((char *,char *, ENVELOPE *, int));
int      test_open      PROTO ((char *,char *, ENVELOPE *, int));
int      test_close     PROTO ((char *,char *, ENVELOPE *, int));
int	 contain	PROTO ((char *,char *));
int	 mystrcmp	PROTO ((char *,char *));
char	*do_trim	PROTO ((char *,char *));
char	*save_to	PROTO ((char *, int, ENVELOPE *));
char	*trim		PROTO ((char *, int, ENVELOPE *));
char	*replace	PROTO ((char *, int, ENVELOPE *));
char	*get_sort_rule	PROTO ((char *, int, ENVELOPE *));
char	*get_index_rule	PROTO ((char *, int, ENVELOPE *));
char	*get_reply_str  PROTO ((char *, int, ENVELOPE *));
char	*get_reply_sub	PROTO ((char *, int, ENVELOPE *));
char	*startup_rule	PROTO ((char *, int, ENVELOPE *));
char	*execute_fcn	PROTO ((char *, char *, int, int));
RULE_ACTION *get_action	PROTO ((char *,int, ENVELOPE *));
char 	*advance_to_char PROTO ((char *, char, int, int *));
char	*get_group	PROTO ((char *));
int	 action_has_function PROTO ((char *, char*));

/* Separators:
 *
 * A separator is a string that separates the rule condition with the rule
 * action. Below is the list of separators
 *
 */

#define  SAVE_TO_SEP  "->"		/* Separator for save_to */
#define  APPLY_SEP    "=>"		/* Separator for do the action */

/*------- Definitions of tokens -------*/
/*------ Keep the list alphabetically sorted, thanks -------*/

#define ADDR_TOKEN   "_ADDRESS_"
#define	CC_TOKEN     "_CC_"
#define FOLDER_TOKEN "_FOLDER_"
#define	FROM_TOKEN   "_FROM_"
#define NICK_TOKEN   "_NICK_"
#define	SEND_TOKEN   "_SENDER_"
#define	SUBJ_TOKEN   "_SUBJECT_"
#define	TO_TOKEN     "_TO_"

#define NTOKENS  8

/* Mail related tokens 
 *
 * The following is an array with the list of tokens.
 */

char* token_rules[] = {
   NICK_TOKEN,
   FROM_TOKEN,
   FOLDER_TOKEN,
   SUBJ_TOKEN,
   TO_TOKEN,
   SEND_TOKEN,
   ADDR_TOKEN,
   CC_TOKEN,
   NULL
};

/*------ Definitions of relational operands -------------*/

typedef struct {
	char       *value;
	int        (*execute)();
} REL_TOKEN;

/* Relational Operands */


#define AND_REL	    "&&"	/* For putting more than one condition 	*/
#define IN_REL      "<<"	/* For belonging relation */
#define	NI_REL      ">>"	/* For contain relation   */
#define	NOT_IN_REL  "!<"	/* Negation of IN_REL	*/
#define	NOT_NI_REL  "!>"	/* Negation of NI_REL	*/
#define	EQ_REL      "=="	/* Test of equality	*/
#define	NOT_EQ_REL  "!="	/* Test of inequality	*/
#define	OPEN_SET    "{"		/* Braces to open a set */
#define	CLOSE_SET   "}"		/* Braces to close a set*/


#define NREL  8

REL_TOKEN rel_rules_test[] = {
   {IN_REL,		test_in},
   {NI_REL,		test_ni},
   {NOT_IN_REL,		test_not_in},
   {NOT_NI_REL,		test_not_ni},
   {EQ_REL,		test_eq},
   {NOT_EQ_REL,		test_not_eq},
   {OPEN_SET,		test_open},
   {CLOSE_SET,		test_close},
   {NULL,		NULL}
};



/*--- Context in which these variables can be used ---*/

typedef struct use_context {
    char        *name;
    int          what_for;
} USE_IN_CONTEXT;


static USE_IN_CONTEXT tokens_use[] = {
    {NICK_TOKEN,	FOR_SAVE},
    {FROM_TOKEN,	FOR_SAVE},
    {TO_TOKEN,		FOR_SAVE},
    {SUBJ_TOKEN,	FOR_SAVE|FOR_FOLDER},
    {SEND_TOKEN,	FOR_SAVE},
    {ADDR_TOKEN,	FOR_SAVE|FOR_FOLDER},
    {CC_TOKEN,		FOR_SAVE},
    {FOLDER_TOKEN,	FOR_SAVE|FOR_FOLDER},
    {NULL,		FOR_NOTHING}
};


typedef struct {
	char         *name;
	char*        (*execute)();
	int	     what_for;
} RULE_FCN;

#define SAVE_FCN	"_SAVE_"
#define TRIM_FCN	"_TRIM_"
#define SORT_FCN	"_SORT_"
#define INDEX_FCN	"_INDEX_"
#define REPLY_FCN	"_REPLY_"
#define RESUB_FCN	"_RESUB_"
#define REPLACE_FCN	"_REPLACE_"
#define STARTUP_FCN	"_STARTUP_"

#define NFCN	8

RULE_FCN rule_fcns[] = {
{SAVE_FCN,	save_to,	FOR_SAVE},
{TRIM_FCN,	trim,		FOR_TRIM},
{REPLACE_FCN,	replace,	FOR_REPLACE},
{SORT_FCN,	get_sort_rule,	FOR_SORT},
{INDEX_FCN,	get_index_rule,	FOR_INDEX},
{REPLY_FCN,	get_reply_str,	FOR_REPLY_INTRO},
{RESUB_FCN,	get_reply_sub,	FOR_RESUB},
{STARTUP_FCN,	startup_rule,	FOR_STARTUP}
};

#define STRICT  0x1
#define RELAXED 0x2
